/*
 * Perform SPI communication using a suitable abstraction augmented with
 * explicit manipulation of a control signal.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/gpio.h>
#include <l4/devices/spi.h>

/* SPI channel abstraction. */

class Spi_hybrid : public Spi_channel_base, public Spi_control_base
{
  Spi_channel_base *_channel;
  Hw::Gpio_chip *_control_device;
  int _control_pin;
  int _control_alt_func;

public:
  explicit Spi_hybrid(Spi_channel_base *channel,
                      Hw::Gpio_chip *control_device, int control_pin,
                      int control_alt_func = -1);

  Spi_channel_base *get_channel();

  void acquire_control(bool asserted);

  void release_control();

  uint32_t send(uint32_t bytes, const uint8_t data[]);

  uint32_t send_dc(uint32_t bytes, const uint8_t data[], const int dc[]);

  uint32_t send_units(uint32_t bytes, const uint8_t data[],
                      uint8_t unit_size, uint8_t char_size);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *spi_hybrid_get_channel(void *channel, void *control_chip, int control_pin,
                             int control_alt_func);

void *spi_hybrid_get_raw_channel(void *channel);

void spi_hybrid_acquire_control(void *channel, int asserted);

void spi_hybrid_release_control(void *channel);

uint32_t spi_hybrid_send(void *channel, uint32_t bytes, const uint8_t data[]);

uint32_t spi_hybrid_send_dc(void *channel, uint32_t bytes, const uint8_t data[],
                            const int dc[]);

uint32_t spi_hybrid_send_units(void *channel, uint32_t bytes, const uint8_t data[],
                               uint8_t unit_size, uint8_t char_size);

EXTERN_C_END
