/*
 * Common SPI abstractions.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

#include <stdint.h>

/* SPI channel abstractions. */

class Spi_channel_base
{
public:
  virtual uint32_t send(uint32_t bytes, const uint8_t data[]) = 0;

  virtual uint32_t send_dc(uint32_t bytes, const uint8_t data[],
                           const int dc[]) = 0;

  virtual uint32_t send_units(uint32_t bytes, const uint8_t data[],
                              uint8_t unit_size, uint8_t char_size) = 0;
};

class Spi_control_base
{
public:
  virtual void acquire_control(bool asserted) = 0;

  virtual void release_control() = 0;
};

#endif /* __cplusplus */
